/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, document, history, tau*/

/**
 * Options page module.
 *
 * @module views/options
 * @requires {@link core/event}
 * @requires {@link models/options}
 * @namespace views/options
 * @memberof views
 */

define({
    name: 'views/options',
    requires: [
        'core/event',
        'models/options'
    ],
    def: function viewsOptions(req) {
        'use strict';

        /**
         * Event module object.
         *
         * @private
         * @type {Module}
         */
        var e = req.core.event,

            /**
             * Options module object.
             *
             * @private
             * @type {Module}
             */
            options = req.models.options,

            /**
             * Page element.
             *
             * @private
             * @type {HTMLElement}
             */
            page = null,

            /**
             * Cancel button element.
             *
             * @private
             * @type {HTMLElement}
             */
            cancelBtn = null,

            /**
             * OK button element.
             *
             * @private
             * @type {HTMLElement}
             */
            okBtn = null,

            /**
             * Input range element.
             *
             * @private
             * @type {HTMLElement}
             */
            range = null,

            /**
             * Preview element.
             *
             * @private
             * @type {HTMLElement}
             */
            preview = null,

            /**
             * White color element.
             *
             * @private
             * @type {HTMLElement}
             */
            white = null,

            /**
             * Red color element.
             *
             * @private
             * @type {HTMLElement}
             */
            red = null,

            /**
             * Green color element.
             *
             * @private
             * @type {HTMLElement}
             */
            green = null,

            /**
             * Blue color element.
             *
             * @private
             * @type {HTMLElement}
             */
            blue = null;

        /**
         * Handles views.main.show.options event.
         *
         * @private
         */
        function show() {
            tau.changePage('#options');
        }

        /**
         * Sets options values.
         *
         * @private
         */
        function setOptionValues() {
            options.setStrokeWidth(range.value);
            options.setStrokeColor(preview.style.backgroundColor);
        }

        /**
         * Sets stroke properties.
         *
         * Expected format of strokeColor is hex.
         *
         * @private
         * @param {number} strokeWidth
         * @param {string} strokeColor
         */
        function setStrokeProperties(strokeWidth, strokeColor) {
            preview.style.height = strokeWidth + 'px';
            preview.style.backgroundColor = strokeColor;
        }

        /**
         * Resets UI.
         *
         * @private
         */
        function resetUI() {
            var strokeWidth = options.getStrokeWidth();

            range.value = strokeWidth;
            setStrokeProperties(strokeWidth, options.getStrokeColor());
        }

        /**
         * Handles pagebeforeshow event.
         *
         * @private
         */
        function onPageBeforeShow() {
            resetUI();
        }

        /**
         * Handles click event on cancel button.
         *
         * @private
         */
        function onCancelBtnTap() {
            history.back();
        }

        /**
         * Handles click event on OK button.
         *
         * @private
         */
        function onOkBtnTap() {
            setOptionValues();
            history.back();
        }

        /**
         * Sets preview element height.
         *
         * @private
         */
        function setPreviewHeight(value) {
            preview.style.height = value + 'px';
        }

        /**
         * Handles change event on range slider.
         *
         * @private
         * @param {Event} ev
         */
        function onRangeChange(ev) {
            setPreviewHeight(ev.target.value);
        }

        /**
         * Handles tap event on white color element.
         *
         * @private
         */
        function onWhiteTap() {
            preview.style.backgroundColor = '#fff';
        }

        /**
         * Handles tap event on red color element.
         *
         * @private
         */
        function onRedTap() {
            preview.style.backgroundColor = '#f00';
        }

        /**
         * Handles tap event on green color element.
         *
         * @private
         */
        function onGreenTap() {
            preview.style.backgroundColor = '#0f0';
        }

        /**
         * Handles tap event on blue color element.
         *
         * @private
         */
        function onBlueTap() {
            preview.style.backgroundColor = '#00f';
        }

        /**
         * Registers view event listeners.
         *
         * @private
         */
        function bindEvents() {
            page.addEventListener('pagebeforeshow', onPageBeforeShow);
            cancelBtn.addEventListener('click', onCancelBtnTap);
            okBtn.addEventListener('click', onOkBtnTap);
            range.addEventListener('change', onRangeChange);
            white.addEventListener('click', onWhiteTap);
            red.addEventListener('click', onRedTap);
            green.addEventListener('click', onGreenTap);
            blue.addEventListener('click', onBlueTap);
        }

        /**
         * Initializes module.
         *
         * @memberof views/options
         * @public
         */
        function init() {
            page = document.getElementById('options');
            cancelBtn = document.getElementById('options-cancel-btn');
            okBtn = document.getElementById('options-ok-btn');
            range = document.getElementById('options-range');
            preview = document.getElementById('options-preview');
            white = document.getElementById('options-colors-white');
            red = document.getElementById('options-colors-red');
            green = document.getElementById('options-colors-green');
            blue = document.getElementById('options-colors-blue');
            bindEvents();
        }

        e.listeners({
            'views.main.show.options': show
        });

        return {
            init: init
        };
    }

});
