/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*global define, tau, document, console*/

/**
 * Exit popup module.
 *
 * @module views/popup
 * @requires {@link core/event}
 * @requires {@link core/systeminfo
 * @requires {@link core/application}
 * @requires {@link core/template}
 * @namespace views/popup
 * @memberof views
 */

define({
    name: 'views/popup',
    requires: [
        'core/event',
        'core/systeminfo',
        'core/application',
        'core/template'
    ],
    def: function popupExit(req) {
        'use strict';

        /**
         * Event module object.
         *
         * @private
         * @type {Module}
         */
        var e = req.core.event,

            /**
             * Systeminfo module object.
             *
             * @private
             * @type {Module}
             */
            sysInfo = req.core.systeminfo,

            /**
             * Template module object.
             *
             * @private
             * @type {Module}
             */
            template = req.core.template,

            /**
             * Application module object.
             *
             * @private
             * @type {Module}
             */
            app = req.core.application;


        /**
         * Creates popup and returns it.
         *
         * @private
         * @param {string} msg
         * @returns {HTMLElement}
         */
        function createPopup(msg) {
            var popupElement = null,
                popupElementCon = document.createElement('div');

            popupElement = template.get('popup', {
                msg: msg
            });
            popupElementCon.innerHTML = popupElement;
            popupElement = popupElementCon.firstElementChild;
            return popupElement;
        }

        /**
         * Handles click event on exit popup OK button.
         *
         * @private
         */
        function onExitPopupOkClick() {
            tau.closePopup();
        }

        /**
         * Handles popuphide event on popup element.
         *
         * @private
         */
        function onPopupHide() {
            try {
                sysInfo.getSystemProperty(
                    'BATTERY',
                    function onValue(battery) {
                        if (!battery.isCharging) {
                            e.fire('battery.charger.off');
                            return;
                        }
                        app.exit();
                    },
                    function onError() {
                        return;
                    }
                );
            } catch (error) {
                console.warn(
                    'Information about battery state cannot be obtained.',
                    error
                );
            }
        }

        /**
         * Opens exit popup.
         *
         * @memberof views/popup
         * @public
         * @param {string} msg
         */
        function openExitPopup(msg) {
            var popupElement = createPopup(msg),
                alertOk = popupElement.querySelector('.alert-ok');

            popupElement.addEventListener('popuphide', onPopupHide);
            alertOk.addEventListener('click', onExitPopupOkClick);
            tau.activePage.appendChild(popupElement);
            tau.openPopup(popupElement);
        }

        return {
            openExitPopup: openExitPopup
        };
    }
});
